import 'package:flutter/material.dart' show Color, TextStyle, Rect, Canvas, Size, CustomPainter, Paint, Gradient, LinearGradient, Alignment, Offset, FilterQuality, Colors, TextSpan, TextPainter, TextDirection;
import 'package:hunterchart/lib/style/chart_style.dart';
import 'model/hunter_chart_model.dart';

double? lastPriceNow = 0;

abstract class BaseChart extends CustomPainter {
  /// linear Gradient
  List<Color> bgColor;
  int rows, columns;
  HunterChartStyle? chartStyle;
  double? spaceText;
  double padding;

  Paint gridPaint = Paint()
    ..isAntiAlias = true
    ..filterQuality = FilterQuality.high
    ..strokeWidth = 0.5
    ..color = Colors.white;

  Paint gridPriceNowPaint = Paint()
    ..isAntiAlias = true
    ..filterQuality = FilterQuality.high
    ..strokeWidth = 1
    ..color = Colors.green;

  BaseChart({required this.bgColor, required this.columns, required this.rows, this.chartStyle, this.spaceText, required this.padding});

  @override
  void paint(Canvas canvas, Size size) {
    initialStyle();
    initChart();
    background(canvas, size);
    textRight(canvas, size);
    chart(canvas, size);
  }

  void initialStyle() {
    gridPaint = Paint()
      ..isAntiAlias = true
      ..filterQuality = FilterQuality.high
      ..strokeWidth = chartStyle!.gridStrokeWidth ?? 0.5
      ..color = chartStyle!.gridColor ?? Colors.white;
  }

  void background(Canvas canvas, Size size) {
    Paint paintBg = Paint();
    Gradient colorBg = LinearGradient(
      begin: Alignment.bottomCenter,
      end: Alignment.topCenter,
      colors: bgColor,
    );
    Rect sizeRectangle = Rect.fromLTRB(0, 0, size.width, size.height);
    canvas.save();
    canvas.drawRect(sizeRectangle, paintBg..shader = colorBg.createShader(sizeRectangle));
    canvas.restore();
  }

  void initChart();
  void textRight(Canvas canvas, Size size);
  void chart(Canvas canvas, Size size);

  void showPriceNow(Offset offset, Canvas canvas, Size size, {String? priceNow}) {
    TextSpan span = TextSpan(text: "${priceNow ?? '0'}", style: TextStyle(color: Colors.white, fontSize: 11));
    TextPainter tp = TextPainter(text: span, textDirection: TextDirection.rtl);
    if (lastPriceNow! < double.parse(priceNow!)) gridPriceNowPaint..color = Colors.green;
    if (lastPriceNow! > double.parse(priceNow)) gridPriceNowPaint..color = Colors.red;
    if (lastPriceNow != double.parse(priceNow)) lastPriceNow = double.parse(priceNow);
    canvas.drawLine(offset, Offset(0, offset.dy), gridPriceNowPaint);
    canvas.drawRect(Rect.fromLTRB(size.width - spaceText! - (padding + 4), offset.dy - 10, size.width, offset.dy + 10), gridPriceNowPaint);
    tp.layout();
    tp.paint(canvas, Offset(size.width - spaceText!, offset.dy - (tp.height - (tp.height / 2))));
  }

  searchingMax(List<HunterChartData> data) {
    double max = 0;
    double min = double.infinity;
    int maxLength = 0;
    for (var i in data) {
      if (max < i.close!) max = i.close!;
      if (min > i.close!) min = i.close!;
      if (maxLength < i.close!.toString().length) maxLength = i.close!.toString().length;
    }
    if (max == min) min -= max - ((max / rows));
    if (maxLength < min.toString().length) maxLength = min.toString().length;
    return {'max': max, 'min': min, 'length': maxLength};
  }

  findY(List x, double value) {
    var a = 0.0, b = 0.0;
    var newRangeY = 0.0, oldRangeY = 0.0;
    double resultY = 0;
    x.reversed.forEach((element) {
      element.forEach((k, v) {
        if (a < k) {
          if (b != a) {
            b = a;
            oldRangeY = newRangeY;
          }
          a = k;
          newRangeY = v;
        }

        if (a >= value && b <= value) {
          resultY = oldRangeY - newRangeY;
          if (value == 250.0) print(newRangeY);
          // diff
          if (a == value)
            resultY = v;
          else if (b == value)
            resultY = oldRangeY;
          else
            resultY = newRangeY + (resultY * (value / a));
          return;
        }
      });
      if (resultY > 0) {
        return;
      }
    });
    return resultY;
  }

  @override
  bool shouldRepaint(BaseChart oldDelegate) {
    return true;
  }
}
