import 'package:hunterchart/lib/model/hunter_chart_model.dart';
import 'package:hunterchart/lib/style/chart_style.dart';

import 'build_chart.dart';
import 'package:flutter/material.dart';

class HunterChart extends StatefulWidget {
  final List<Color>? bgColor;
  final int rows, columns;
  final List<HunterChartData>? data;
  final HunterChartStyle? chartStyle;

  HunterChart({Key? key, this.bgColor, this.rows = 4, this.columns = 4, this.data, this.chartStyle}) : super(key: key);

  @override
  _HunterChartState createState() => _HunterChartState();
}

class _HunterChartState extends State<HunterChart> {
  late BuildChart chart;
  bool onDrag = false;
  double dragOffset = 0, startDragOffset = 0, currentDragOffset = 0;

  @override
  Widget build(BuildContext context) {
    chart = BuildChart(
      bgColor: widget.bgColor ?? [Color(0xff000000), Color(0xff000000)],
      rows: widget.rows,
      columns: widget.columns,
      data: widget.data!,
      chartStyle: widget.chartStyle,
      dragOffset: currentDragOffset,
    );

    return GestureDetector(
      child: CustomPaint(
        size: Size(double.infinity, double.infinity),
        painter: chart,
      ),
      onHorizontalDragStart: (drag) {
        if (!onDrag) onDrag = true;
        startDragOffset = drag.globalPosition.dx;
      },
      onHorizontalDragUpdate: (drag) {
        dragOffset = startDragOffset - drag.globalPosition.dx;
        currentDragOffset -= dragOffset * 0.01;
        if (currentDragOffset < 0) currentDragOffset = 0;
        setState(() {});
      },
      onHorizontalDragEnd: (drag) {
        if (onDrag) onDrag = false;
        startDragOffset = 0;
        dragOffset = 0;
      },
    );
  }
}
